/*
 * Hash abstract data type.
 * Hash
 */

#ifndef __HASH_H
#define __HASH_H


/* Definition des types de donnee */
typedef struct Hash *Hash;
typedef char *HashIdent;

typedef enum {
  HASH_OK,
  HASH_ERROR,
  HASH_DOUBLON,
  HASH_NOFOUND
} HashStatus;



Hash hash_new(int hash_size, int ident_size);
/*
 * Creation d'une hash table vide. 
 */
HashStatus hash_destroy(Hash hash);
/*
 * Destruction d'une table.
 */


HashStatus hash_add(Hash hash, char *ident, void *data, int data_size);
/*
 * Ajoute une entree dans la hash table. La taille
 * de la donne est variable. La longueur de l'identificateur
 * est fixee  la creation de la table.
 */
HashStatus hash_supp(Hash hash, char *ident);
/*
 * Supression d'une cellule.
 */


int hash_size(Hash hash, char *ident);
/*
 * Retourne la taille d'une cellule.
 */


HashStatus hash_data(Hash hash, char *ident, void *data, int *data_size);
/*
 * Retourne le contenu d'une cellule.
 */
HashStatus hash_all(Hash hash, void *param,
		    int (*proc)(void *param, char *ident,
				void *data, int data_size));
/*
 * Applique a toutes les cellules une fonction.
 */


HashStatus hash_save(Hash hash, char *filename);
/*
 * Sauve une hash table dans un fichier.
 */
Hash hash_load(char *filename);
/*
 * Charge une hash table a partir d'un fichier.
 */


#endif /* ___HASH_H */
