/* 
 * LittleBlackBook - Address Book
 *
 * Authors: Navarro Mikael "mickey" (mikael.navarro@libertysurf.fr)
 *	
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "zlib.h"

#ifdef GETOPT
#include <unistd.h>
#include <sys/utsname.h>
#include <getopt.h> /* filtrage de la ligne de commmande */
#else
#include "getopt/getopt.h"
#define GETOPT
#endif


/* Lecture d'un fichier LBB -> Ecriture d'un fichier LDIF (netscape) */
int lbb2ldif(char *filename) {
  FILE *file;
  gzFile *gzfile;

  char line[1024], *fin;
  char *tks="|\n";        /* tokens */
  int section=0;    /* section courrante */

  char lastname[32], firstname[32], nickname[16];
  char society[32], function[32];
  char street1[64], street2[64], zipcode[8], city[16], state[16];
  char homephone[16], faxphone[16], cellphone[16], workphone[16];
  char email1[48], email2[48], email3[48], url[64];
  char comments[256];
  char lastrevision[16];
  char publickeypgp[128];


  /* Ouverture du fichier LBB (conpression level 9) */
  if(!(gzfile=gzopen(filename, "rb9"))) {
    fprintf(stderr, "Can't open lbb file(LBB) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Loading lbb file : %s\n", filename);
  /* Ouverture du fichier LDIF */
  if(!(file=fopen(strcat(filename, ".ldif"), "wb"))) {
    fprintf(stderr, "Can't open ldif file(netscape) : %s\n", filename);
    gzclose(gzfile);
    exit(-2);
  }
  fprintf(stdout, "Writing ldif file : %s", filename);

  /* Lecture des options de config. */
  while((fin=gzgets(gzfile, line, 1024))!=NULL) {
    if(!strncmp(line, ">> ", 3)) {
      section += 1;
      /* Lecture d'une carte LBB */
      strcpy(lastname, strtok(line+3, tks));
      strcpy(firstname, strtok(NULL, tks));
      strcpy(nickname, strtok(NULL, tks));

      gzgets(gzfile, line, 1024);
      strcpy(society, strtok(line, tks));
      strcpy(function, strtok(NULL, tks));

      gzgets(gzfile, line, 1024);
      strcpy(street1, strtok(line, tks));
      strcpy(street2, strtok(NULL, tks));
      strcpy(zipcode, strtok(NULL, tks));
      strcpy(city, strtok(NULL, tks));
      strcpy(state, strtok(NULL, tks));

      gzgets(gzfile, line, 1024);
      strcpy(homephone, strtok(line, tks));
      strcpy(workphone, strtok(NULL, tks));
      strcpy(cellphone, strtok(NULL, tks));
      strcpy(faxphone, strtok(NULL, tks));

      gzgets(gzfile, line, 1024);
      strcpy(email1, strtok(line, tks));
      strcpy(email2, strtok(NULL, tks));
      strcpy(email3, strtok(NULL, tks));
      strcpy(url, strtok(NULL, tks));

      gzgets(gzfile, line, 1024);
      strcpy(comments, strtok(line, tks));
      strcpy(lastrevision, strtok(NULL, tks));
      strcpy(publickeypgp, strtok(NULL, tks));

      /* Ecriture de la carte dans le fichier ldif (netscape) */
      fprintf(file, "dn: cn=%s %s,mail=%s\n", lastname, firstname, 
	      (strncmp(email1, " ", 1))? email1 : " ");
      fprintf(file, "modyfimestamp: 20000520080541Z\n"); /* un numero bidon ! */
      fprintf(file, "cn: %s %s\n", lastname, firstname);
      if(strncmp(nickname, " ", 1)) fprintf(file, "xmozillanickname: %s\n", nickname);
      if(strncmp(email1, " ", 1)) fprintf(file, "mail: %s\n", email1);
      fprintf(file, "xmozillausehtmlmail : TRUE\n");
      if(strncmp(city, " ", 1))	fprintf(file, "locality: %s\n", city);
      fprintf(file, "givenname: %s\n", firstname);
      fprintf(file, "sn: %s\n", lastname);
      if(strncmp(street1, " ", 1)) fprintf(file, "streetaddress: %s", street1);
      if(strncmp(street2, " ", 1)) fprintf(file, "; %s", street2);
      if(strncmp(street1, " ", 1)) fprintf(file, "\n");
      if(strncmp(zipcode, " ", 1)) fprintf(file, "postalcode: %s\n", zipcode);
      if(strncmp(state, " ", 1)) fprintf(file, "countryname: %s\n", state);
      if(strncmp(homephone, " ", 1)) fprintf(file, "homephone: %s\n", homephone);
      if(strncmp(cellphone, " ", 1)) fprintf(file, "cellphone: %s\n", cellphone);
      if(strncmp(url, " ", 1)) fprintf(file, "homeurl: %s\n", url);
      if(strncmp(workphone, " ", 1)) fprintf(file, "xmozillaanyphone: %s\n", workphone);
      if(strncmp(comments, " ", 1)) fprintf(file, "description: %s\n", comments);
      if(strncmp(society, " ", 1)) fprintf(file, "o: %s\n", society);
      if(strncmp(function, " ", 1)) fprintf(file, "title: %s\n", function);
      if(strncmp(workphone, " ", 1)) fprintf(file, "telephonenumber: %s\n", workphone);
      if(strncmp(faxphone, " ", 1)) fprintf(file, "facsimiletelephonenumber: %s\n", faxphone);
      fprintf(file, "\n");
    }
  } /* -> while */

  /* Fermeture du fichier LDIF */
  fclose(file);
  /* Fermeture du fichier LBB */
  gzclose(gzfile);
  fprintf(stdout, "\n");  

  return 1;
}


static int print_usage(void) {
  printf("\nUsage: lbb2ldif file.lbb.gz [--help] [--version]\n");
  printf("lbb2ldif - Convert AddressBook.\n\n");
#ifdef GETOPT
  printf("  Options:\n");
  printf("  --version      output version information and exit\n");
  printf("  --help         display this message and exit\n");
#endif
  printf("\nReport bugs to mnavarro@caramail.com or mickael.navarro@libertysurf.fr\n");
  return 0;
}

int main(int argc, char **argv) {
#ifdef GETOPT
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'r'},
    {0, 0, 0, 0}
  };
#endif

#ifdef GETOPT
  /* Filtrage des options de la ligne de commande */
  while((c=getopt_long_only(argc, argv, "rh", 
			    long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 'h':
      print_usage(); return 0;
    case 'r':
      printf("%s %.1f\n", argv[0], 1.0); return 0;
    default:
      break;
    }
  }
  /* Traitement des autres arguments */
  if(optind < argc) {
    lbb2ldif(argv[optind]);
  }
  else {
    /* Pas assez d'arguments */
    fprintf(stderr, "Too few argument : %s\n", argv[0]);
  }
#else 
  /* No line command option :( */
  /* Some default value */
#endif

  return 0;
}


