/* 
 * LittleBlackBook - Address Book
 *
 * Authors: Navarro Mikael "mickey" (mikael.navarro@libertysurf.fr)
 *	
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "zlib.h"

#ifdef GETOPT
#include <unistd.h>
#include <sys/utsname.h>
#include <getopt.h> /* filtrage de la ligne de commmande */
#else
#include "getopt/getopt.h"
#define GETOPT
#endif


/* Lecture d'un fichier LDIF (netscape) -> Ecriture d'un fichier LBB */
int ldif2lbb(char *filename) {
  FILE *file;
  gzFile *gzfile;

  char line[1024], *fin;
  char *tks="\n";   /* tokens */
  int section=0;    /* section courrante */

  char lastname[32], firstname[32], nickname[16];
  char society[32], function[32];
  char street1[64], zipcode[8], city[16], state[16];
  char homephone[16], faxphone[16], cellphone[16], workphone[16];
  char email1[32], url[64];
  char comments[256];
  char lastrevision[16];

  time_t tps = time(NULL); /* date et heure calandaire */
  struct tm *tu = gmtime(&tps); /* Temps Universel (TU) */

  /* Pretraitement de la date et de l'heure */
  sprintf(lastrevision, "%d/%d/%d", tu->tm_mday, 1+tu->tm_mon, 1900+tu->tm_year);

  /* Ouverture du fichier LDIF */
  if(!(file=fopen(filename, "rb"))) {
    fprintf(stderr, "Can't open ldif file(netscape) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Loading ldif file : %s\n", filename);
  /* Ouverture du fichier LBB (conpression level 9) */
  if(!(gzfile=gzopen(strcat(filename, ".lbb.gz"), "wb9"))) {
    fprintf(stderr, "Can't open lbb file(LBB) : %s\n", filename);
    fclose(file);
    exit(-2);
  }
  fprintf(stdout, "Writing lbb file : %s", filename);

  /* Initialisations */
  strcpy(lastname, " "); strcpy(firstname, " "); strcpy(nickname, " ");
  strcpy(society, " "); strcpy(function, " ");
  strcpy(street1, " "); strcpy(zipcode, " "); strcpy(city, " "); strcpy(state, " ");
  strcpy(homephone, " "); strcpy(faxphone, " "); strcpy(cellphone, " "); strcpy(workphone, " ");
  strcpy(email1, " "); strcpy(url, " ");
  strcpy(comments, " ");

  /* Lecture des options de config. */
  while((fin=fgets(line, 1024, file))!=NULL) {
    if(!strncmp(line, "dn:", 3)) {
      section += 1;
      /*gzprintf(gzfile, "\n");*/
      if(section > 1) {
	/* Ecriture de la section */
	gzprintf(gzfile, ">> %s|%s|%s\n", lastname, firstname, nickname);
	gzprintf(gzfile, "%s|%s\n", society, function);
	gzprintf(gzfile, "%s|%s|%s|%s|%s\n", street1, " ",
		 zipcode, city, state);
	gzprintf(gzfile, "%s|%s|%s|%s|%s|%s\n", homephone, workphone,
		 cellphone, faxphone, " ", " ");
	gzprintf(gzfile, "%s|%s|%s|%s\n", email1, " ",
		 " ", url);
	gzprintf(gzfile, "%s|%s|", comments,
		 lastrevision);
	gzprintf(gzfile, "%s\n", " ");
	/* Reinitialisations */
	strcpy(lastname, " "); strcpy(firstname, " "); strcpy(nickname, " ");
	strcpy(society, " "); strcpy(function, " ");
	strcpy(street1, " "); strcpy(zipcode, " "); strcpy(city, " "); strcpy(state, " ");
	strcpy(homephone, " "); strcpy(faxphone, " "); strcpy(cellphone, " "); strcpy(workphone, " ");
	strcpy(email1, " "); strcpy(url, " ");
	strcpy(comments, " ");
      }
    }
    else if(!strncmp(line, "modyfimestamp: ", 15));
    else if(!strncmp(line, "cn: ", 4));
    else if(!strncmp(line, "xmozillanickname: ", 18)) strcpy(nickname, strtok(line+18, tks));
    else if(!strncmp(line, "mail: ", 6)) strcpy(email1, strtok(line+6, tks));
    else if(!strncmp(line, "xmozillausehtmlmail: ", 21));
    else if(!strncmp(line, "locality: ", 10)) strcpy(city, strtok(line+10, tks));
    else if(!strncmp(line, "givenname: ", 11)) strcpy(firstname, strtok(line+11, tks)); 
    else if(!strncmp(line, "sn: ", 4)) strcpy(lastname, strtok(line+4, tks)); 
    else if(!strncmp(line, "streetaddress: ", 15)) strcpy(street1, strtok(line+15, "\n"));
    else if(!strncmp(line, "postalcode: ", 12)) strcpy(zipcode, strtok(line+12, tks));
    else if(!strncmp(line, "countryname: ", 13)) strcpy(state, strtok(line+13, tks));
    else if(!strncmp(line, "homephone: ", 11)) strcpy(homephone, strtok(line+11, tks));
    else if(!strncmp(line, "cellphone: ", 11)) strcpy(cellphone, strtok(line+11, tks));
    else if(!strncmp(line, "homeurl: ", 9)) strcpy(url, strtok(line+9, tks));
    else if(!strncmp(line, "xmozillaanyphone: ", 18)) strcpy(workphone, strtok(line+18, tks));
    else if(!strncmp(line, "description: ", 13)) strcpy(comments, strtok(line+13, tks));
    else if(!strncmp(line, "o: ", 3)) strcpy(society, strtok(line+3, tks));
    else if(!strncmp(line, "st: ", 4));
    else if(!strncmp(line, "title: ", 7)) strcpy(function, strtok(line+7, tks));
    else if(!strncmp(line, "telephonenumber: ", 17)) strcpy(workphone, strtok(line+17, tks));
    else if(!strncmp(line, "facsimiletelephonenumber: ", 26)) strcpy(faxphone, strtok(line+26, tks));
    else if(!strncmp(line, "pagerphone: ", 12));
    else if(!strncmp(line, "ou: ", 4));
    else if(!strncmp(line, "objectclass: ", 13));
    else ;
  } /* -> while */

  /* Ecriture de la derniere section */
  gzprintf(gzfile, ">> %s|%s|%s\n", lastname, firstname, nickname);
  gzprintf(gzfile, "Org: %s|%s\n", society, function);
  gzprintf(gzfile, "Address: %s|%s|%s|%s|%s\n", street1, " ",
	   zipcode, city, state);
  gzprintf(gzfile, "Phone: %s|%s|%s|%s|%s|%s\n", homephone, workphone,
	   cellphone, faxphone, " ", " ");
  gzprintf(gzfile, "Internet: %s|%s|%s|%s\n", email1, " ",
	   " ", url);
  gzprintf(gzfile, "Notes: %s|%s|", comments,
	   lastrevision);
  gzprintf(gzfile, "%s\n", " ");

  /* Fermeture du fichier LBB */
  gzclose(gzfile);
  /* Fermeture du fichier LDIF */
  fclose(file);
  fprintf(stdout, "\n");  

  return 1;
}


static int print_usage(void) {
  printf("\nUsage: ldif2lbb file.ldif [--help] [--version]\n");
  printf("ldif2lbb - Convert AddressBook.\n\n");
#ifdef GETOPT
  printf("  Options:\n");
  printf("  --version      output version information and exit\n");
  printf("  --help         display this message and exit\n");
#endif
  printf("\nReport bugs to mnavarro@caramail.com or mickael.navarro@libertysurf.fr\n");
  return 0;
}

int main(int argc, char **argv) {
#ifdef GETOPT
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'r'},
    {0, 0, 0, 0}
  };
#endif

#ifdef GETOPT
  /* Filtrage des options de la ligne de commande */
  while((c=getopt_long_only(argc, argv, "rh", 
			    long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 'h':
      print_usage(); return 0;
    case 'r':
      printf("%s %.1f\n", argv[0], 1.0); return 0;
    default:
      break;
    }
  }
  /* Traitement des autres arguments */
  if(optind < argc) {
    ldif2lbb(argv[optind]);
  }
  else {
    /* Pas assez d'arguments */
    fprintf(stderr, "Too few argument : %s\n", argv[0]);
  }
#else 
  /* No line command option :( */
  /* Some default value */
#endif

  return 0;
}
