/* 
 * LittleBlackBook - Address Book
 *
 * Authors: Navarro Mikael "mickey" (mikael.navarro@libertysurf.fr)
 *	
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#ifdef GETOPT
#include <unistd.h>
#include <sys/utsname.h>
#include <getopt.h> /* filtrage de la ligne de commmande */
#else
#include "getopt/getopt.h"
#define GETOPT
#endif


/* Mise en minuscule */
static char *wtolower(char *s) {
  int i;
  char *ts;

  ts=(char*)malloc(sizeof(char)*(strlen(s)+1));

  for(i=0; s[i]!='\0'; i++)
    ts[i]=tolower(s[i]);
  ts[i]='\0';

  return ts;
}

/* Mise en majuscule */
static char *wtoupper(char *s) {
  int i;
  char *ts;

  ts=(char*)malloc(sizeof(char)*(strlen(s)+1));

  for(i=0; s[i]!='\0'; i++)
    ts[i]=toupper(s[i]);
  ts[i]='\0';

  return ts;
}

/* Lecture d'un fichier LDIF (netscape) -> Ecriture d'un LaTeX */
int ldif2tex(char *filename) {
  FILE *file;
  FILE *texfile;

  char line[1024], *fin;
  char *tks="\n";   /* tokens */
  int section=0;    /* section courrante */

  char lastname[32], firstname[32], nickname[16];
  char org[32], title[32];
  char street1[64], street2[64], zipcode[8], city[16], state[16];
  char homephone[16], workphone[16], cellular[16], fax[16], pager[16], otherphone[16];
  char email1[32], email2[32], email3[32], url[64];
  char comments[256];
  char lastrevision[16], pgpkey[128];

  time_t tps = time(NULL); /* date et heure calandaire */
  struct tm *tu = gmtime(&tps); /* Temps Universel (TU) */

  /* Pretraitement de la date et de l'heure */
  sprintf(lastrevision, "%d/%d/%d", tu->tm_mday, 1+tu->tm_mon, 1900+tu->tm_year);

  /* Ouverture du fichier LDIF */
  if(!(file=fopen(filename, "rb"))) {
    fprintf(stderr, "Can't open ldif file(netscape) : %s\n", filename);
    exit(-2);
  }
  fprintf(stdout, "Loading ldif file : %s\n", filename);
  /* Ouverture du fichier LaTeX */
  if(!(texfile=fopen(strcat(filename, ".tex"), "wb"))) {
    fprintf(stderr, "Can't open tex file(LaTeX) : %s\n", filename);
    fclose(file);
    exit(-2);
  }
  fprintf(stdout, "Writing tex file : %s", filename);

  /* Initialisations */
  strcpy(lastname, ""); strcpy(firstname, ""); strcpy(nickname, "");
  strcpy(org, ""); strcpy(title, "");
  strcpy(street1, ""); 
  strcpy(zipcode, ""); strcpy(city, ""); strcpy(state, "");
  strcpy(homephone, ""); strcpy(workphone, "");  strcpy(cellular, ""); 
  strcpy(fax, "");
  strcpy(email1, ""); strcpy(url, "");
  strcpy(comments, "");

  /* LaTeX header */
  fprintf(texfile, "\\documentclass[a4paper,10pt]{article}\n\n");
  fprintf(texfile, "\\usepackage[latin1]{inputenc}\n");
  fprintf(texfile, "\\usepackage[T1]{fontenc}\n");
  fprintf(texfile, "\\usepackage[francais]{babel}\n\n");

  fprintf(texfile, "\\pagestyle{empty}\n\n");
  fprintf(texfile, "\\title{gLBB - LittleBlackBook}\n");
  fprintf(texfile, "\\author{ Mikael 'mickey' Navarro}\n\n");

  fprintf(texfile, "\\begin{document}\n\n");

  fprintf(texfile, "%%\\maketitle\n\n");
  fprintf(texfile, "gLBB - LittleBlackBook \\\\ ");
  fprintf(texfile, "Mikael 'mickey' Navarro \\\\ ");
  fprintf(texfile, "\\begin{em}\n%s\n\\end{em} \\\\ \n\n", filename);

  fprintf(texfile, "\\baselineskip 10 pt\n\n");

  /*fprintf(texfile, "\\parindent = 0pt\n");
    fprintf(texfile, "\\parskip = 0pt\n");
    fprintf(texfile, "\\voffset = -3.5cm");
    fprintf(texfile, "\\vsize = 19cm");
    fprintf(texfile, "\\hoffset = -1cm");
    fprintf(texfile, "\\hsize = 20cm");*/

  /* Lecture des options de config. */
  while((fin=fgets(line, 1024, file))!=NULL) {
    if(!strncmp(line, "dn:", 3)) {
      section += 1;
      if(section > 1) {
	/* Ecriture de la section */
	fprintf(texfile, "\n\\vspace{0.5cm}\n");
	fprintf(texfile, "\\begin{tabular}{|p{4cm}p{6cm}|}\n");
	fprintf(texfile, "\\hline\n");
	fprintf(texfile, "\\bf{%s} & \\bf{%s} \\\\ \n", wtoupper(lastname), firstname);
	fprintf(texfile, "\\hline\n");
	if(strlen(nickname)) {
	  fprintf(texfile, "nickname: & \\emph{%s} \\\\ \n", nickname);
	  fprintf(texfile, "\\hline\n");
	}

	if(strlen(org) || strlen(title)) {
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, "\\bf{Society} & \\\\ \n");
	  fprintf(texfile, "\\hline\n");
	  if(strlen(org))   fprintf(texfile, "Organization: & \\emph{%s} \\\\ \n", org);
	  if(strlen(title)) fprintf(texfile, "Title: & \\emph{%s} \\\\ \n", title);
	  fprintf(texfile, "\\hline\n");
	}

	if(strlen(street1) || strlen(zipcode) || strlen(city) || strlen(state)) {
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, "\\bf{Address} &\\\\ \n");
	  fprintf(texfile, "\\hline\n");
	  if(strlen(street1)) fprintf(texfile, "Street1: & \\emph{%s} \\\\ \n", street1);
	  if(strlen(zipcode)) fprintf(texfile, "Zipcode: & \\emph{%s} \\\\ \n", zipcode);
	  if(strlen(city))    fprintf(texfile, "City & \\emph{%s} \\\\ \n", city);
	  if(strlen(state))   fprintf(texfile, "State & \\emph{%s} \\\\ \n", state);
	  fprintf(texfile, "\\hline\n");
	}

	if(strlen(homephone) || strlen(workphone) || strlen(cellular) || strlen(fax)) {
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, "\\bf{Phone} & \\\\ \n");
	  fprintf(texfile, "\\hline\n");
	  if(strlen(homephone)) fprintf(texfile, "Home: & \\emph{%s} \\\\ \n", homephone);
	  if(strlen(workphone)) fprintf(texfile, "Work: & \\emph{%s} \\\\ \n", workphone);
	  if(strlen(cellular))  fprintf(texfile, "Cellular: & \\emph{%s} \\\\ \n", cellular);
	  if(strlen(fax))       fprintf(texfile, "Fax: & \\emph{%s} \\\\ \n", fax);
	  fprintf(texfile, "\\hline\n");
	}

	if(strlen(email1) || strlen(url)) {
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, "\\bf{Internet} & \\\\ \n");
	  fprintf(texfile, "\\hline\n");
	  if(strlen(email1)) fprintf(texfile, "Email1: & \\emph{%s} \\\\ \n", email1);
	  if(strlen(url))    fprintf(texfile, "Url: & \\emph{%s} \\\\ \n", url);
	  fprintf(texfile, "\\hline\n");
	}

	if(strlen(comments)) {
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, "\\bf{Comments} & \\\\ \n");
	  fprintf(texfile, "\\hline\n");
	  fprintf(texfile, " & \\emph{%s} \\\\ \n", comments);
	  fprintf(texfile, "\\hline\n");
	}
	
	fprintf(texfile, "\\end{tabular}\n");

	/* Reinitialisations */
	strcpy(lastname, ""); strcpy(firstname, ""); strcpy(nickname, "");
	strcpy(org, ""); strcpy(title, "");
	strcpy(street1, ""); 
	strcpy(zipcode, ""); strcpy(city, ""); strcpy(state, "");
	strcpy(homephone, ""); strcpy(workphone, "");  strcpy(cellular, ""); 
	strcpy(fax, "");
	strcpy(email1, ""); strcpy(url, "");
	strcpy(comments, "");
      }
    }
    else if(!strncmp(line, "modyfimestamp: ", 15));
    else if(!strncmp(line, "cn: ", 4));
    else if(!strncmp(line, "xmozillanickname: ", 18)) strcpy(nickname, strtok(line+18, tks));
    else if(!strncmp(line, "mail: ", 6)) strcpy(email1, strtok(line+6, tks));
    else if(!strncmp(line, "xmozillausehtmlmail: ", 21));
    else if(!strncmp(line, "locality: ", 10)) strcpy(city, strtok(line+10, tks));
    else if(!strncmp(line, "givenname: ", 11)) strcpy(firstname, strtok(line+11, tks)); 
    else if(!strncmp(line, "sn: ", 4)) strcpy(lastname, strtok(line+4, tks)); 
    else if(!strncmp(line, "streetaddress: ", 15)) strcpy(street1, strtok(line+15, "\n"));
    else if(!strncmp(line, "postalcode: ", 12)) strcpy(zipcode, strtok(line+12, tks));
    else if(!strncmp(line, "countryname: ", 13)) strcpy(state, strtok(line+13, tks));
    else if(!strncmp(line, "homephone: ", 11)) strcpy(homephone, strtok(line+11, tks));
    else if(!strncmp(line, "cellphone: ", 11)) strcpy(cellular, strtok(line+11, tks));
    else if(!strncmp(line, "homeurl: ", 9)) strcpy(url, strtok(line+9, tks));
    else if(!strncmp(line, "xmozillaanyphone: ", 18)) strcpy(workphone, strtok(line+18, tks));
    else if(!strncmp(line, "description: ", 13)) strcpy(comments, strtok(line+13, tks));
    else if(!strncmp(line, "o: ", 3)) strcpy(org, strtok(line+3, tks));
    else if(!strncmp(line, "st: ", 4));
    else if(!strncmp(line, "title: ", 7)) strcpy(title, strtok(line+7, tks));
    else if(!strncmp(line, "telephonenumber: ", 17)) strcpy(workphone, strtok(line+17, tks));
    else if(!strncmp(line, "facsimiletelephonenumber: ", 26)) strcpy(fax, strtok(line+26, tks));
    else if(!strncmp(line, "pagerphone: ", 12));
    else if(!strncmp(line, "ou: ", 4));
    else if(!strncmp(line, "objectclass: ", 13));
    else ;
  } /* -> while */

  /* Ecriture de la derniere section */
  fprintf(texfile, "\n\\vspace{0.5cm}\n");
  fprintf(texfile, "\\begin{tabular}{|p{4cm}p{6cm}|}\n");
  fprintf(texfile, "\\hline\n");
  fprintf(texfile, "\\bf{%s} & \\bf{%s} \\\\ \n", wtoupper(lastname), firstname);
  fprintf(texfile, "\\hline\n");
  if(strlen(nickname)) {
    fprintf(texfile, "nickname: & \\emph{%s} \\\\ \n", nickname);
    fprintf(texfile, "\\hline\n");
  }

  if(strlen(org) || strlen(title)) {
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, "\\bf{Society} & \\\\ \n");
    fprintf(texfile, "\\hline\n");
    if(strlen(org))   fprintf(texfile, "Organization: & \\emph{%s} \\\\ \n", org);
    if(strlen(title)) fprintf(texfile, "Title: & \\emph{%s} \\\\ \n", title);
    fprintf(texfile, "\\hline\n");
  }

  if(strlen(street1) || strlen(zipcode) || strlen(city) || strlen(state)) {
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, "\\bf{Address} &\\\\ \n");
    fprintf(texfile, "\\hline\n");
    if(strlen(street1)) fprintf(texfile, "Street1: & \\emph{%s} \\\\ \n", street1);
    if(strlen(zipcode)) fprintf(texfile, "Zipcode: & \\emph{%s} \\\\ \n", zipcode);
    if(strlen(city))    fprintf(texfile, "City & \\emph{%s} \\\\ \n", city);
    if(strlen(state))   fprintf(texfile, "State & \\emph{%s} \\\\ \n", state);
    fprintf(texfile, "\\hline\n");
  }

  if(strlen(homephone) || strlen(workphone) || strlen(cellular) || strlen(fax)) {
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, "\\bf{Phone} & \\\\ \n");
    fprintf(texfile, "\\hline\n");
    if(strlen(homephone)) fprintf(texfile, "Home: & \\emph{%s} \\\\ \n", homephone);
    if(strlen(workphone)) fprintf(texfile, "Work: & \\emph{%s} \\\\ \n", workphone);
    if(strlen(cellular))  fprintf(texfile, "Cellular: & \\emph{%s} \\\\ \n", cellular);
    if(strlen(fax))       fprintf(texfile, "Fax: & \\emph{%s} \\\\ \n", fax);
    fprintf(texfile, "\\hline\n");
  }

  if(strlen(email1) || strlen(url)) {
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, "\\bf{Internet} & \\\\ \n");
    fprintf(texfile, "\\hline\n");
    if(strlen(email1)) fprintf(texfile, "Email1: & \\emph{%s} \\\\ \n", email1);
    if(strlen(url))    fprintf(texfile, "Url: & \\emph{%s} \\\\ \n", url);
    fprintf(texfile, "\\hline\n");
  }

  if(strlen(comments)) {
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, "\\bf{Comments} & \\\\ \n");
    fprintf(texfile, "\\hline\n");
    fprintf(texfile, " & \\emph{%s} \\\\ \n", comments);
    fprintf(texfile, "\\hline\n");
  }
	
  fprintf(texfile, "\\end{tabular}\n");

  /* End LaTex file */
  fprintf(texfile, "\n");
  fprintf(texfile, "\\end{document}\n\n");

  /* Fermeture du fichier LaTeX */
  fclose(texfile);
  /* Fermeture du fichier LDIF */
  fclose(file);
  fprintf(stdout, "\n");  

  return 1;
}


static int print_usage(void) {
  printf("\nUsage: ldif2tex file.ldif [--help] [--version]\n");
  printf("ldif2tex - Convert AddressBook.\n\n");
#ifdef GETOPT
  printf("  Options:\n");
  printf("  --version      output version information and exit\n");
  printf("  --help         display this message and exit\n");
#endif
  printf("\nReport bugs to mikael.navarro@libertysurf.fr\n");
  return 0;
}

int main(int argc, char **argv) {
#ifdef GETOPT
  int c, opt_ind=0;
  static struct option long_options[] = {
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'r'},
    {0, 0, 0, 0}
  };
#endif

#ifdef GETOPT
  /* Filtrage des options de la ligne de commande */
  while((c=getopt_long_only(argc, argv, "rh", 
			    long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 'h':
      print_usage(); return 0;
    case 'r':
      printf("%s %.1f\n", argv[0], 1.0); return 0;
    default:
      break;
    }
  }
  /* Traitement des autres arguments */
  if(optind < argc) {
    ldif2tex(argv[optind]);
  }
  else {
    /* Pas assez d'arguments */
    fprintf(stderr, "Too few argument : %s\n", argv[0]);
  }
#else 
  /* No line command option :( */
  /* Some default value */
#endif

  return 0;
}
